body {
  font-size: 12px;
}

.table th,
.table td {
  padding: 4px;
  line-height: 16px;
}

.row-fluid [class*="span"] {
    min-height: 24px;
}

select {
  height: 24px;
  padding: 4px 6px;
  margin-bottom: 4px;
  font-size: 12px;
  line-height: 16px;
}

.form-horizontal select {
  margin-bottom: 4px;
}

textarea,
input[type="text"],
input[type="password"],
input[type="datetime"],
input[type="datetime-local"],
input[type="date"],
input[type="month"],
input[type="time"],
input[type="week"],
input[type="number"],
input[type="email"],
input[type="url"],
input[type="search"],
input[type="tel"],
input[type="color"],
.uneditable-input {
  height: 14px;
  padding: 4px 6px;
  margin-bottom: 4px;
  font-size: 12px;
  line-height: 16px;
}

label, input, button, select, textarea {
    font-size: 12px;
    font-weight: normal;
    line-height: 16px;
}

.btn-group > .btn,
.btn-group > .dropdown-menu,
.btn-group > .popover {
  font-size: 12px;
}

.btn-group > .btn-mini {
  font-size: 9.5px;
}

.btn-group > .btn-small {
  font-size: 10.9px;
}

.btn-group > .btn-large {
  font-size: 15.5px;
}

.form-horizontal .control-group {
    margin-bottom: 0px;
    margin-top: 0px;
}

.pager {
    padding: 0px;
}

.pager-input {
    height: 16px;
    margin-top: 4px;
    text-align: center;
}

.btn {
    font-size: 12px;
    line-height: 18px;
}

.input-button {
    padding: 1px 4px;
    line-height: 20px;
}

h1,
h2,
h3 {
  line-height: 36px;
}

h1 {
  font-size: 36.5px;
}

h2 {
  font-size: 29.5px;
}

h3 {
  font-size: 22.5px;
}

h4 {
  font-size: 15.5px;
}

h5 {
  font-size: 12px;
}

h6 {
  font-size: 11.9px;
}

h1 small {
  font-size: 22.5px;
}

h2 small {
  font-size: 15.5px;
}

h3 small {
  font-size: 12px;
}

h4 small {
  font-size: 12px;
}

.input-integer {
  width: 30%;
}

.input-float {
  width: 35%;
}

.input-currency {
  width: 40%;
}

.input-date {
  width: 40%;
}

.input-date-time {
  width: 55%;
}

.input-select {
  width: %60;
}

.input-with-buttons {
  width: 100%;
}

.input-item {
  width: 66%;
}

.input-text {
  width: 80%;
}

.input-blob {
  width: 100%;
}

h4.title {
    margin: 6px 0;
}

/*
.form-horizontal .control-group {
    margin-bottom: 10px;
    margin-top: 10px;
}

.input-button {
    padding: 4px 6px
}

.pager {
    padding: 4px;
}
*/

.template {
    display: none;
}

/*
.item-btn {
    margin: 1px 1px;
    width: 100px;
}

.item-btn-with-hint {
    margin: 1px 1px;
    width: 144px;
}
*/

.inline {
    display:inline;
}

.modal.fade {
    -webkit-transition: opacity 0.1s linear;
       -moz-transition: opacity 0.1s linear;
         -o-transition: opacity 0.1s linear;
            transition: opacity 0.1s linear;
}

.inner-table {
    outline: none;
}

.table-bordered  {
    border-left: 1px solid #dddddd;
}

.table-bordered th:first-child,
.table-bordered td:first-child {
    border-left: 0;
}

.outer-table th {
    border-bottom: 1px solid #dddddd;
}

.table tbody tr.selected > td:not(.field-selected) {
    background-color: #e5e5e5;
}

.table tbody tr.selected-focused > td:not(.field-selected-focused) {
    background-color: #d9edf7;
}

.table tbody tr td.selected {
    background-color: #e5e5e5;
}

.table tbody td.field-selected {
    background-color: #cacaca;
}

.table tbody td.field-selected-focused {
    background-color: #b4e3fa;
}

.modal-backdrop,
.modal-backdrop.fade.in{
    opacity: 0.2;
    filter: alpha(opacity=20);
    background: #fff;
}

.table th{
    color:#7F7F7F;
}

.table td,
.table th {
    cursor: default;
    -webkit-user-select: none;
    -moz-user-select: none;
    -khtml-user-select: none;
    -ms-user-select: none;

}

.tooltip-inner {
  max-width: 400px;
  padding: 8px;
  color: #ffffff;
  text-align: center;
  text-decoration: none;
  background-color: #b94a48;
  -webkit-border-radius: 4px;
     -moz-border-radius: 4px;
          border-radius: 4px;
}

.top-margin {
  margin-top: 6px;
}

.top-border {
  border-top: 1px solid #eee;
}

.bottom-border {
  border-bottom: 1px solid #eee;
}

.right-border {
  border-right: 1px solid #eee;
}

.left-border {
  border-left: 1px solid #eee;
}

.normal-modal-border {
    border-bottom: 1px solid #eee;
}

.error-modal-border {
    border: 1px solid red;
}

.selection-box {
    position:    absolute;
    background: transparent;
    border:     1px dotted #999;
}

.datepicker {
  font-size: 12px;
}

.treeview {
   outline: none;
}

.treeview li div {
    white-space: nowrap;
}

.treeview li div {
    cursor: default;
    -webkit-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    -o-user-select: none;
    -khtml-user-select: none;
    user-select: none;
}

.treeview li.parent div span.tree-bullet {
    font-family: "Courier New", Courier, monospace;
    font-weight: bold;
    cursor: pointer;
}

.treeview li.selected > div span.tree-text {
    background-color: #e5e5e5;
}

.treeview li.selected-focused > div span.tree-text {
    background-color: #d9edf7;
}

.treeview li div span.tree-text {
    cursor: default;
    padding: 4px ;
}

.treeview ul:first-child {
    margin-left: 0px;
}
